/*****************************************************************************
                                                  Author: Farasat A.S. Bokhari
                                               Last Modified: November/11/2014

Purpose:
  This program reads in SAS dataset training.simulateddrugs01 and estimates
  a LA/AIDS model on a subsegment of drugs (drugs 4,5,6,7) in the segment
  MPH-IR.  The model provides both the SUR and 3SLS estimates.
  After estimating the model, this program also (i) computes Marshallian
  and Hicksian (conditional) elasticities matrices (and prints them) and, (ii)
  re-prints the demand parameters in nice format using SAS's IML procedures.

  Information on all the input dataset is documented in the PDF file
  readme-data-simulateddrugs01-sas7bdat.pdf.  Please refer to the readme file
  for information on the variables.

Output(s):
  All relevant results are saved in the HTML file:
  "results-AIDS-on-MPH-ER.html"

Main Steps of the Program:

  STEP 1:  Read in the main data set training.simulateddrugs01 and from it
  create a smaller and cleaner temporary dataset work.segment122. This data set
  contains all the variables necessary to estimate the LA/AIDS model.

  STEP 2:  Regression coefficients (from SUR & 3SLS) are saved in temporary
  datasets (i) m1f2results and m1f2resultsB  (the first one contains regression
  coefficients related to price and expenditure variables in each equation where
  as the second one contains estimates related to coefficients on other
  exogenous variables in the model such as log of children, log of doctors etc.
  in an area.

  STEP 3:  Read in the coefficients related to price and expenditure variables
  in m1f2results into SAS's matrix language (via proc IML) along with shares at
  the sample mean  and use the conditional elasticities formulas to compute and
  print Marshallian and Hicksian elasticities.  Also compute the expenditure
  elasticities and, given  that the model was estimated using restrictions in
  STEP 1, verify Cournot & Engle Aggregations and Hicks Law.  Repeat this process
  using SUR estimates and then using 3SLS estimates.

  STEP 4 (unnecessary):  Read in all the regression coefficients and their
  variances in m1f2results and m1f2resultsB and print them in a cleaner way using
  SAS's IML where coefficients from multiple equations are aligned up.  Send the
  results to the the same output file.

Note:  Upon exiting SAS all temporary datasets will be deleted.


/*****************************************************************************/


libname training 'I:\bokhari\fb_research\analysis\adhd\training\data\';
filename myfile1 "I:\bokhari\fb_research\analysis\adhd\training\progs\00results-AIDS-on-MPH-ER.html";

/*Handle myfile1 gives the path and name of HTML file where results will be saved */ 

/***************************** STEP 1 ****************************************/

data segment122 ;
   set training.simulateddrugs01
   (keep = fips year s4-s7 so4-so7 lpo1-lpo17 lpoz1-lpoz17 
           lpoi2 lpoii2  lpzi2 lpzii2 y2 t1 t2
           lnpop lnkids lnmds lncaiddrugs lnmcaidenrollees pcpi
           msac stabr cenregc cenregn cendivc cendivn cntyst poptot
	       msapmsa99c 
		   r4-r7 po4-po7 poz1-poz17
);
   if year < 2000 then delete;
   rename
   y2     = y   	
   lpoi2  = lpi		lpoii2 = lpii
   lpzi2  = lpzi 	lpzii2 = lpzii

   so4    = s01		so5    = s02
   so6    = s03		so7    = s04
   s4    = s1		s5    = s2
   s6    = s3		s7    = s4
   r4    = r1		r5    = r2
   r6    = r3		r7    = r4
   po4    = p1		po5    = p2
   po6    = p3		po7    = p4
  ;

   lp1   = lpo4;   lp2   = lpo5;
   lp3   = lpo6;   lp4   = lpo7;
   lpz1  = lpoz4;  lpz2  = lpoz5;
   lpz3 =  lpoz6;  lpz4 =  lpoz7;

attrib _all_ label=' ';
run; quit;

data segment122;
   set segment122;
   ly = log(y);
   lypi = ly - lpi;
   lypzi = ly - lpzi;
   t1ly = t1*ly;
   t2ly = t2*ly;
run; quit;


/***************************** STEP 2 ****************************************/

ods html newfile=proc;
proc model data=segment122 plots = none print;
    /*Omit Last Share Equation  for adding up restrictions */
    s1 = a1 + b1*lypi +  c11*t1 + c12*t2 + c13*lnkids + c14*lnmds + c15*lncaiddrugs + c16*lnmcaidenrollees +
         g11*lp1 + g12*lp2 + g13*lp3 + g14*lp4 ;

    s2 = a2 + b2*lypi + c21*t1  + c22*t2 + c23*lnkids + c24*lnmds + c25*lncaiddrugs + c26*lnmcaidenrollees +
         g21*lp1 + g22*lp2 + g23*lp3 + g24*lp4 ;

    s3 = a3 + b3*lypi + c31*t1 + c32*t2 + c33*lnkids + c34*lnmds + c35*lncaiddrugs + c36*lnmcaidenrollees  +
         g31*lp1 + g32*lp2 + g33*lp3 + g34*lp4 ;
    fit s1 s2 s3 / sur 3sls hausman  converge = .00001;
    /* Homogenity restrictions  */
    restrict g11 + g12 + g13 + g14  = 0;
    restrict g21 + g22 + g23 + g24  = 0;
    restrict g31 + g32 + g33 + g34  = 0;
   /*Symmetry restrictions */
    restrict g12 = g21;
    restrict g13 = g31;
    restrict g23 = g32;
	/* Save estimated coefficients and covariances on prices and expenditures in work.m1f2results*/
	/* This is so thata we can later on print the coefficients or call them up in a different */
	/* procedure to estimate elasticities */
    estimate b1, g11, g12, g13, g14,
             b2, g21, g22, g23, g24,
             b3, g31, g32, g33, g34,
            'b4' -(b1+b2+b3),
            'g41' g14,
            'g42' g24,
			'g43' g34,
            'g44' -(g14 + g24 + g34),
     /outest=m1f2results outcov;
     /* Save estimated coefficients and covariances on exogenous variables in work.m1f2resultsB*/
     estimate c11, c12, c13, c14, c15, c16,
			  c21, c22, c23, c24, c25, c26,
              c31, c32, c33, c34, c35, c36,
			 'c41' - (c11 + c21 + c31),
			 'c42' - (c12 + c22 + c32),
             'c43' - (c13 + c23 + c33),
			 'c44' - (c14 + c24 + c34),
			 'c45' - (c15 + c25 + c35),
			 'c46' - (c16 + c26 + c36),
     /outest=m1f2resultsb outcov ;
   endogenous s1-s3 lp1-lp4 lypi ;
   instruments  lpz1 lpz2 lpz3 lpz4 lpzi
                 t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
                 ly t1ly t2ly;
   run; quit ;

/***************************** STEP 3 *****************************************
Compute (& print) conditional elasticities computed at sample mean 
Computations are done via the matrix language IML. But first prepare the 
datasets with me mean values as well as those containing model parameters.

******************************************************************************/

/*First create a data set containing values of shares at which elasticities
will be computed */


proc means data = segment122 noprint ;
 var s1-s4 s01-s04;
 class year;
 output out = M1F2shares mean = s1-s4 s01-s04;
 run; quit;


data M1F2shares (drop = _type_ _freq_ year);
  set M1F2shares;
  if year = . then output;
run; quit;

/*Create a dataset with names of drugs (for printing later) */
data namesegment122;
infile datalines dsd dlm='=';
input brand $ description $50.;
datalines ;
Brand01 = Ritalin SR,LA
Brand02 = Metadate ER,CD
Brand04 = Methylin ER
Brand05 = Generic Meth SR
;
run; quit;



ods html file = myfile1 newfile = none;
title1 'Conditional and Unconditioanl Elasticties (3SLS/SUR) at Mean Shares';
title2 'Created via SAS program <estimate-AIDS-on-MPH-ER.sas>';

/******************************  SUR ELASTICITIES ****************************/
%let results = m1f2results;          /* <-- name of dataset with model parameters */
%let model = 'SUR';                  /* <--  set as 3SLS or SUR */
%let bnames = namesegment122;        
%let atshares = M1F2shares;          /* <-- name of data set with mean shares     */


    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2 b3 b4} into b;
    b = b`;   					  /*b is a column vector */
    read all var{g11 g12 g13 g14} into g1;
    read all var{g21 g22 g23 g24} into g2;
	read all var{g31 g32 g33 g34} into g3;
	read all var{g41 g42 g43 g44} into g4;
	g = g1//g2//g3//g4;
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02 s03 s04} into s0;
	read all var{s1 s2 s3 s4} into s;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2', 'Brand3', 'Brand4'};
    brandsp = brands`;

    one = i(4);

	
    s0b = s0@b;              /*Note: b is a column vector and s is a row vector*/
    sp = s`;                                
    emij = (g - s0b)#(1/sp) - one;               /*Marshallian Elasticity */
    eta = 1 + b/sp;                              /*Expenditure Elasticity */
    etap = eta`;                                 /*Expenditure Elasticity as a row*/
    ehij = emij + s@eta;                        /*Compensated (Hicksian) Elasticity*/
    haij = ehij#(1/s);                          /*Hicks-Allen measures of elasticity */

    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;     /*define print module*/
    print key;
    print chk1;
    print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;

	run print;       /*execute print module */

	quit;

/*************************** 3SLS ELASTICITIES *******************************/
/*note the repetitive nature of the code ... this can be set as a macro *****/
/*The code above can be set as macro and all we would need to change is *****/
/*one word ... from SUR to 3SLS. for now, i just copy/paste and re-run  *****/
/*the entire code for elasticitis with just that one change.            *****/
/*****************************************************************************/ 


%let results = m1f2results;
%let model = '3SLS';          /* <--  set as 3SLS or SUR */
%let bnames = namesegment122;
%let atshares = M1F2shares;


    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2 b3 b4} into b;
    b = b`;   					  /*b is a column vector */
    read all var{g11 g12 g13 g14} into g1;
    read all var{g21 g22 g23 g24} into g2;
	read all var{g31 g32 g33 g34} into g3;
	read all var{g41 g42 g43 g44} into g4;
	g = g1//g2//g3//g4;
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02 s03 s04} into s0;
	read all var{s1 s2 s3 s4} into s;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2', 'Brand3', 'Brand4'};
    brandsp = brands`;

    one = i(4);

	
    s0b = s0@b;                 /*Note: b is a column vector and s is a row vector*/
    sp = s`;                                
    emij = (g - s0b)#(1/sp) - one;               /*Marshallian Elasticity */
    eta = 1 + b/sp;                              /*Expenditure Elasticity */
    etap = eta`;                                 /*Expenditure Elasticity as a row*/
    ehij = emij + s@eta;                        /*Compensated (Hicksian) Elasticity*/
    haij = ehij#(1/s);                          /*Hicks-Allen measures of elasticity */

    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshallian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;     /*define print module*/
    print key;
    print chk1;
    print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;

	run print;       /*execute print module */

	quit;

/***************************  THE END  (OPTIONAL CODE BELOW )******************/
/* The code below is optional .... it goes back extract all the estimated    */
/* coefficients and standard errors and prints them in the same HTML file    */
/* in a nicer form using SAS's IML routine.  As such this entire section can  */
/* be commented/blocked out.                                                  */
/* One objectiv is to show how to write a macro for a repetitve task ... as   */
/* could have been done for the elasticities above.                           */
/*****************************************************************************/

/**************** STEP 4: REPRINT MODEL COEFFICIENTS IN A NICE WAY ***********/
/*DEFINE MACRO called PARMSE*/

%macro parmse(indata, outdata, mod);

data foo (drop = _status_);
  set &indata;
run; quit;


proc iml;
  use foo;
  read all var{_NAME_} into names where  (_TYPE_ = 'SUR' & _NAME_ ^= '');
  read all var _NUM_ into vsur where (_TYPE_ = 'SUR' & _NAME_ ^= '');
  read all var _NUM_ into viv where (_TYPE_ = '3SLS' & _NAME_ ^= '');
  read all var _NUM_ into bsur where (_TYPE_ = 'SUR' & _NAME_ = '');
  read all var _NUM_ into biv where (_TYPE_ = '3SLS' & _NAME_ = '');
  close foo;


  k = ncol(vsur);
  nt = vsur[1,1];
  /**********************************************************************************
    Set STATES = 48 if data was demeaned by states before regressions were estimated.
    else set STATES = 0;
  *********************************************************************************/
  *** states = 48;
  states = 0;

  vsur = vsur[,2:k] ;
  viv = viv[,2:k] ;
  bsur = bsur[,2:k];
  biv = biv[,2:k];
  k = ncol(vsur);

  sdsur = sqrt(vecdiag(vsur)*((nt - k)/(nt - states - k)));
  sdiv = sqrt(vecdiag(viv)*((nt - k)/(nt - states - k)));

  out = bsur//sdsur`//biv//sdiv`;
  rownames =  {"SUR - Beta" "SUR - SD" "3SLS - Beta" "3SLS - SD"};
  mattrib out colname = names
              rowname = rownames;
  *print out;

   create work.foo from out [colname = names];
   append from out;
   close work.foo;

   type = {'SUR' 'PARM', 'SUR' 'SE', '3SLS' 'PARM', '3SLS' 'SE'};
   create work.label from type[colname = {'_TYPE_' 'STAT'}];
   append from type;
   close work.label;


quit;

data &outdata;
  model = &mod;
  merge label foo;
run; quit;

proc datasets lib=work nolist;
delete label foo;
run; quit;
%mend;
/************************* END OF MACRO PARMSE **************************/

%parmse(m1f2results,m1f2resultsx, 'MPH-ER' );
%parmse(m1f2resultsb,m1f2resultsbx, 'MPH-ER');
data m1f2resultsx ;  merge m1f2resultsx m1f2resultsbx; run; quit;


data modelbetase ;
   length model _type_ stat $8. ;
   length b1 b2 b3 b4
          g11 g12 g13 g14
          g21 g22 g23 g24
          g31 g32 g33 g34
          g41 g42 g43 g44
          c11 c12 c13 c14 c15 c16 c17
          c21 c22 c23 c24 c25 c26 c27
          c31 c32 c33 c34 c35 c36 c37
          c41 c42 c43 c44 c45 c46 c47  8.;

 set  m1f2resultsx ;
 rename _type_ = type;


run; quit;

proc iml;



 use modelbetase;
 read all where (model = 'MPH-ER' & type = 'SUR');
 close modelbetase;

  s1 = b1//g11//g12//g13//g14//c13//c14//c15//c16//c11//c12//c17;
  s2 = b2//g21//g22//g23//g24//c23//c24//c25//c26//c21//c22//c27;
  s3 = b3//g31//g32//g33//g34//c33//c34//c35//c36//c31//c32//c37;
  s4 = b4//g41//g42//g43//g44//c43//c44//c45//c46//c41//c42//c47;
  mpherols = s1||s2||s3||s4;

   colnames2 = {"s1 (Ritalin SR/LA" "s2 (Metadate ER/CD)" "s3 (Methylin ER)" "s4 (Generics)"};
   rownames2 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnp3" "" "lnp4" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" "" "timeCb" ""};
   rownames2 = rownames2`;

  mattrib mpherols rowname = rownames2
            colname = colnames2
            label = "Segment MPH-ER (M1F2) -- SUR Estimates"
            format = 5.3;

 print mpherols;



 use modelbetase;
 read all where (model = 'MPH-ER' & type = '3SLS');
 close modelbetase;

  s1 = b1//g11//g12//g13//g14//c13//c14//c15//c16//c11//c12//c17;
  s2 = b2//g21//g22//g23//g24//c23//c24//c25//c26//c21//c22//c27;
  s3 = b3//g31//g32//g33//g34//c33//c34//c35//c36//c31//c32//c37;
  s4 = b4//g41//g42//g43//g44//c43//c44//c45//c46//c41//c42//c47;
  mpheriv = s1||s2||s3||s4;

   colnames2 = {"s1 (Ritalin SR/LA" "s2 (Metadate ER/CD)" "s3 (Methylin ER)" "s4 (Generics)"};
   rownames2 = {"ln(R/P)" "" "lnp1" "" "lnp2" "" "lnp3" "" "lnp4" "" "lnkids" "" "lnmds" "" "lncaiddrugs" "" "lncaidenrollees" "" "time" "" "timeSq" "" "timeCb" ""};
   rownames2 = rownames2`;

  mattrib mpheriv rowname = rownames2
            colname = colnames2
            label = "Segment MPH-ER (M1F2) -- 3SLS Estimates"
            format = 5.3;

 print mpheriv;



quit;
