/*****************************************************************************
                                                  Author: Farasat A.S. Bokhari
                                               Last Modified: November/12/2015

Purpose:
  This program reads in SAS dataset training.simulateddrugs01 and estimates
  a LES model on a subsegment of drugs (drugs 4,5,6,7) in the segment
  MPH-IR.  The model provides both the system estimates with NLS and NL3LS 
  estimates  (i.e. what SAS class Nonlinear SUR and Nonlinear 3SLS.

  After estimating the model, this program also (i) computes Marshallian
  and Hicksian (conditional) elasticities matrices (and prints them) using 
  SAS's IML routine.   

  Information on all the input dataset is documented in the PDF file
  readme-data-simulateddrugs01-sas7bdat.pdf.  Please refer to the readme file
  for information on the variables.

Output(s): Some of the regresion results and the elasticity matrices are 
are saved in the HTML file "results-LES-on-MPH-ER.html"

Main Steps of the Program:

  STEP 1:  Read in the main data set training.simulateddrugs01 and from it
  create a smaller and cleaner temporary dataset work.segment122. This data set
  contains all the variables necessary to estimate the LES demand model.

  STEP 2:  Regression coefficients (from nonlinear-SUR & nonlinear-3SLS) are
  sent to the  output HTML file.  In addition, primary coefficients related to 
  the LES model -- ie those that can interpreted as alphas and betas in the 
  standard LES model are saved in a temporary dataset m1f2results.  These can
  then be called up later to compute elasticities (or to just reprint them). 

  STEP 3:  Read in the estimated coefficients from m1f2results into SAS's matrix
  language (via proc IML) along with shares and prices at the sample mean
  and use them to compute conditional elasticities.  Also compute the expenditure
  elasticities and, given  that the model was estimated using restrictions in
  STEP 1, verify Cournot & Engle Aggregations and Hicks Law.  Repeat this process
  using NL-SUR estimates or NL-3SLS estimates.


********************************************************************************/


libname training 'I:\bokhari\fb_research\analysis\adhd\training\data\';
filename myfile1 "I:\bokhari\fb_research\analysis\adhd\training\progs\results-LES-on-MPH-ER.html";


/*Handle myfile1 gives the path and name of HTML file where results will be saved */ 

/***************************** STEP 1 ****************************************/

data segment122 ;
   set training.simulateddrugs01
   (keep = fips year s4-s7 so4-so7 lpo1-lpo17 lpoz1-lpoz17 
           lpoi2 lpoii2  lpzi2 lpzii2 y2 t1 t2
           lnpop lnkids lnmds lncaiddrugs lnmcaidenrollees pcpi
           msac stabr cenregc cenregn cendivc cendivn cntyst poptot
	       msapmsa99c
		   r4-r7 po4-po7 poz1-poz17
);

   if year < 2000 then delete;

   rename
   y2     = y   	
   lpoi2  = lpi		lpoii2 = lpii
   lpzi2  = lpzi 	lpzii2 = lpzii

   so4    = s01		so5    = s02
   so6    = s03		so7    = s04
   s4    = s1		s5    = s2
   s6    = s3		s7    = s4

   r4    = r1		r5    = r2
   r6    = r3		r7    = r4

   po4    = p1		po5    = p2
   po6    = p3		po7    = p4
  ;


   lp1   = lpo4;   lp2   = lpo5;
   lp3   = lpo6;   lp4   = lpo7;

   lpz1  = lpoz4;  lpz2  = lpoz5;
   lpz3 =  lpoz6;  lpz4 =  lpoz7;

attrib _all_ label=' ';
run; quit;

data segment122;
   set segment122;
   ly = log(y);
   lypi = ly - lpi;
   lypzi = ly - lpzi;
   t1ly = t1*ly;
   t2ly = t2*ly;
run; quit;


/***************************** STEP 2 ****************************************/
ods html file = myfile1 newfile = none;
title1 'Created via SAS program <estimate-LES-on-MPH-ER.sas>';

proc model data = segment122 plots=none;

parms a1 a2 a3 a4 b1 b2 b3 b4;
r1 = a1*p1 + b1*(y -  a1*p1 - a2*p2 - a3*p3 - a4*p4)
       + c10  + c11*t1 + c12*t2 + c13*lnkids + c14*lnmds + c15*lncaiddrugs + c16*lnmcaidenrollees ;
r2 = a2*p2 + b2*(y -  a1*p1 - a2*p2 - a3*p3 - a4*p4)
       + c20    + c21*t1 + c22*t2 + c23*lnkids + c24*lnmds + c25*lncaiddrugs + c26*lnmcaidenrollees;
r3 = a3*p3 + b3*(y -  a1*p1 - a2*p2 - a3*p3 - a4*p4)
       + c30    + c31*t1 + c32*t2 + c33*lnkids + c34*lnmds + c35*lncaiddrugs + c36*lnmcaidenrollees ;
r4 = a4*p4 + b4*(y -  a1*p1 - a2*p2 - a3*p3 - a4*p4)
        + c40  + c41*t1 + c42*t2 + c43*lnkids + c44*lnmds + c45*lncaiddrugs + c46*lnmcaidenrollees;

fit r1 r2 r3 /sur 3sls  ;

 estimate  a1, a2, a3, a4,
           b1, b2, b3, 'b4'  1 - (b1 + b2 + b3),
		   /outest=lesresults outcov ;


   instruments  poz4 poz5 poz6 poz7
   t1 t2 lnkids lnmds lncaiddrugs lnmcaidenrollees
   y;


run; quit;

proc means data = segment122 noprint ;
 var s1-s4 s01-s04 p1-p4 y;
 class year;
 output out = M1F2shares mean = s1-s4 s01-s04 p1-p4 y;
 run; quit;


data M1F2shares (drop = _type_ _freq_ year);
  set M1F2shares;
  if year = . then output;
run; quit;

/*Create a dataset with names of drugs (for printing later) */
data namesegment122;
infile datalines dsd dlm='=';
input brand $ description $50.;
datalines ;
Brand01 = Ritalin SR,LA
Brand02 = Metadate ER,CD
Brand04 = Methylin ER
Brand05 = Generic Meth SR
;
run; quit;



title2 'Elasticties (NL-3SLS) at Mean Shares';


/******************************  *** ELASTICITIES ****************************/
%let results = lesresults;
%let model = '3SLS';          /* <--  set as 3SLS or SUR ****/
%let bnames = namesegment122;
%let atshares = M1F2shares;


    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;



    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2 b3 b4} into b;
    read all var{a1 a2 a3 a4} into a;
    b = b`;  a = a`; 					  			/* a, b are column vectors */
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02 s03 s04} into s0;
	read all var{s1 s2 s3 s4} into s;
	read all var{p1 p2 p3 p4} into p;
	read all var{y} into y;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2', 'Brand3', 'Brand4'};
    brandsp = brands`;

    one = i(4);
    sp = s`;                         /* Mean shares as column s-prime*/
    phi  = - 1*(1-  (1/y)*p*a);      /*phi is a scalar */
	eta  = b/sp;             		 /*Expenditure Elasticity --- column vector*/
	etap = eta`;                     /* Expenditure Elasticity as a row old eep*/
    emij = one#eta*phi - eta*(sp#(1 + (phi*eta)))`;
    ehij = emij + s@eta;             /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);               /*Hicks-Allen measures of elasticity */
	                                 /* or use  ehij = one#eta*phi - eta*(s#etap)*phi; */


    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';
  mattrib emij label = {"Conditional Marshallian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;     /*define print module*/
    print key;
    print chk1;
    print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;

	run print;   
		quit;

