/******************************************************************************
										 		       Author: Farasat Bokhari
                                                   Last Updated: Dec/27/2008


Program Name: macro-elasticities-ver05.sas

Purpose: This program defines a series of macros named

            elasticityXXb(arg#1,arg#2,arg#3,arg#4,arg#5,arg#6)

where XX defines a different macro. For instance if XX = 3, then it defines a
macro named elasticity3b(arg#1,arg#2,arg#3,arg#4,arg#5,arg#6).
All the macros defined in this program are very similar but the main
difference is between the number of drugs within the segment. Thus

    elasticity2b --> estimates elasticities for a segment with 2 brands
    elasticity3b --> estimates elasticities for a segment with 3 brands
    elasticity4b --> estimates elasticities for a segment with 4 brands
    elasticity5b --> estimates elasticities for a segment with 5 brands


Each macro computes (conditional) Marshallian and Hicksian elasticities (as
well as expenditure elasticities) for all the drugs in the segment,
conditional on total segment expenditures. Also, it stores the elasticities 
(as different matrices) in a SAS catalog (named in arg#5) as well as 
single row dataset (also named in arg#5). Elasticities can be computed using
either the OLS estimates or the 2SLS estimates (via the value of the arg#4).
Also, the elasticities need to be computed at some specified values of the 
shares and associated values of the stone weights. As such, these can be any
values but if in arg#3 we name the dataset work.m1f1shares (which was created
by the some earlier macro), then the elasticities are computed at the values
s01,s02,...etc. (stone weights) and s1,s2,...etc. (shares). The macro assumes
that parameter estimates have been computed and stored via the PROC MODEL 
statement in some earlier step (or via an earlier macro).

The arguments of the macros are as follows:
        arg#1 --> Name of a dataset set that contains the actual names of the
                  drugs in the segment.  Thus, the dataset must contain 2
                  variables called "brand" and "description" and should have
                  as many rows as the number of drugs in the segment.
        arg#2 --> Name of dataset with parameter estimates of the AIDS estimation
                  via the PROC MODEL statement
        arg#3 --> Name of a dataset with 1 row and columns labeled s01,s02,..., i.e.
                  values of stone-weights as well as s1,s2,..., the shares
                 (at which the elasticity should be computed)
        arg#4 --> Name of the estimation method (such as 'OLS' or '2SLS' etc.) so
                  that parameters from such a method can be read in from the
                  dataset named in arg#2.
        arg#5 --> Name of a SAS catalog where several matrices will be
                  stored.  Additionally, a dataset with the same name 
                 will be created (as a single row) with all the elasticities
                 stored in it. Specifically, Marshallian, Hicksian and Expenditure
                 (in that order).  Thus, if there were three brands, then the
                 length of the dataset will be (3*3) + (3*3) + 3 =21.
        arg#6 --> Valid values are 'print' or 'noprint' in which case the 
                 elasticities will be printed or not.

        Examples:
        a. elasticity3b(namesegment101,m1f1results,meanshares,'OLS',m1f1ols,noprint)
        b. elasticity3b(namesegment101,m1f1results,meanshares,'2SLS',m1f1iv,print)

        In both the examples above, the elasticities are computed for a segment
        that has a total of three drugs in it but in the first example, the
        macro computes elasticities based on OLS estimation of AIDS
        parameters and stores some computed matrices (elasticities etc.) in a
        catalog called work.m1f1ols but does not print the results. Additionally,
        it also creates a temp dataset wor.m1f1ols with 21 variables a 1 row. However,
        in the second example it uses the 2SLS parameter estimates and stores the
        computed matrices in a  catalog called work.m1f1iv and in a dataset called
        work.m1f1iv.  Additionally, it also prints the results.

        Note the following (for both the examples):
        1. work.namesegment101 is the name of a dataset that contains the
        actual names of the drugs in the segment. These must be in a variable
        called 'description' and MUST BE in the order of the drugs. Thus, the
        first row of the dataset must correspond to the name of the drug whose
        share is called s1. The second row must be the name of the drug that
        whose share is called s2 and so on.
        2. The dataset work.m1f1results contains the OLS and 2SLS parameters
        in the format generated by the PROC MODEL procedure
        3. The output catalogs work.m1f1ols OR work.m1f1iv (in these two
        examples) stores the following matrices:
           Let J = Number of Drugs/Brands in the segment
           B --> J by 1: Contains AIDS model coefficient on ln(R/P)
           G --> J by J: Contains AIDS model coefficients on lnPi
           S --> 1 by J: Contains the shares at with elasticities were
                   computed
           S0 --> 1 by J: Contains the Stone-weigts used in the elasticity
                   computations
           Ehij --> J by J: Hicksian elasticities
           Emij --> J by J: Marshallian elsaticities
           Etap --> 1 by J: Expenditure elasticities
           Brands --> J by 1: Contains text, 'Brand1',...,'BrandJ'
           Key  --> J by 2: Brand1, Name of the Drug
                            Brand2, Name of the Drug
                            ... etc.
                            Essentially matrix version of the dataset
                            work.namesegment101

******************************************************************************/




/*****************************************************************************/
/**************************** DEFINE MACRO ELASTICITY2B **********************/
/*****************************************************************************/

/*******************************************************************************

Name of Macro:          elasticity2b(bnames,results,atshares,model,matcat,print)


Purpose: The purpose of the macro is to compute the matrix of marshallian and
compensated elasticities for a segment with three (2) drugs in it.


Arguments of Macro:
(1) &bnames
    Datasset containing the names and descriptions of the drugs in this segment
    e.g. namesegment110 etc.)

(2) &results
    Name of the dataset containing estimation results from AIDs model where the
    results are informat as stored by "PROC MODEL" procedure. Assumption
    is that the dataset will contain both the coefficients and the var-cov matrix.
    Additionally, the datset contains a column "_TYPE_" (= Type of Estimation) which
    states what type of estimation procedure was used (OLS, 2SLS, ITSUR etc.).
    Results from multiple estimation procedures may be stored in the same dataset.

(3) &atshares
    Name of the dataset containing the value of the shares (one per drug) at which
    elasticities should be computed (e.g. meanshares). Additionally, same dataset
    contains the values of the stone-weigts. 

(4) &model
    Argument indicating elasticities to be based on which model, i.e., ols, 2sls,
    etc. Note that the value of the macro must be stated in quotes and in upper case
    as in 'OLS' or '2SLS' or 'ITSUR' ect.

(5) &matcat
    Name of a SAS catalog where the results can be stored. Also, name of the temp
    SAS dataset where all results will be sent (in one row).

(6) &PRINT 
     Valid values are print or noprint. 

*********************************************************************************/


%MACRO ELASTICITY2B(BNAMES,RESULTS,ATSHARES,MODEL,MATCAT,PRINT);

    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2} into b;
    b = b`;   					  /*b is a column vector */
    read all var{g11 g12} into g1;
    read all var{g21 g22} into g2;
	g = g1//g2;
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02} into s0;
	read all var{s1 s2} into s;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2'};
    brandsp = brands`;

    one = i(2);
   

    s0b = s0@b;              /*Note: b is a column vector and s is a row vector*/
    sp = s`;
    emij = (g - s0b)#(1/sp) - one;           /*Marshallian Elasticity old eta1*/
    eta = 1 + b/sp;                   /*Expenditure Elasticity old ee and eep*/
    etap = eta`;                   /* Expenditure Elasticity as a row old eep*/
    ehij = emij + s@eta;         /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);            /*Hicks-Allen measures of elasticity */


    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;
    print key;
    print chk1;
    print etap2;
    print emij ;
    print ehij;
	print haij;
    finish;
   

    start noprint;
	zzz = 5;  *dummy step for no printing module;
	finish;

    run &print;

    /*  save the marshallian elasticity
    create mpe from emij[colname = brands];
    append from emij;
    close mpe;
    */
    
    elasticities = shape(emij,1)||shape(ehij,1)||etap||shape(haij,1);
    enames = {m11 m12 m21 m22
              h11 h12 h21 h22
              e1 e2 
              a11 a12 a21 a22};
    create &matcat from elasticities[colname = enames];
    append from elasticities;
    close &matcat;


    RESET Storage=work.&matcat;
    remove _all_;
    store b g s s0 key brands emij ehij etap haij;

    quit;

    proc datasets lib = work nolist;
    delete coeffs varcov;
    run; quit;

	%MEND;



/*****************************************************************************/
/**************************** DEFINE MACRO ELASTICITY3B **********************/
/*****************************************************************************/

/*******************************************************************************

Name of Macro:          elasticity3b(bnames,results,atshares,model,matcat,print)


Purpose: The purpose of the macro is to compute the matrix of marshallian and
compensated elasticities for a segment with three (3) drugs in it.


Arguments of Macro:
(1) &bnames
    Datasset containing the names and descriptions of the drugs in this segment
    e.g. namesegment110 etc.)

(2) &results
    Name of the dataset containing estimation results from AIDs model where the
    results are informat as stored by "PROC MODEL" procedure. Assumption
    is that the dataset will contain both the coefficients and the var-cov matrix.
    Additionally, the datset contains a column "_TYPE_" (= Type of Estimation) which
    states what type of estimation procedure was used (OLS, 2SLS, ITSUR etc.).
    Results from multiple estimation procedures may be stored in the same dataset.

(3) &atshares
    Name of the dataset containing the value of the shares (one per drug) at which
    elasticities should be computed (e.g. meanshares). Additionally, same dataset
    contains the values of the stone-weigts. 

(4) &model
    Argument indicating elasticities to be based on which model, i.e., ols, 2sls,
    etc. Note that the value of the macro must be stated in quotes and in upper case
    as in 'OLS' or '2SLS' or 'ITSUR' ect.

(5) &matcat
    Name of a SAS catalog where the results can be stored. Also, name of the temp
    SAS dataset where all results will be sent (in one row).

(6) &PRINT 
     Valid values are print or noprint. 

*********************************************************************************/


%MACRO ELASTICITY3B(BNAMES,RESULTS,ATSHARES,MODEL,MATCAT,PRINT);

    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2 b3} into b;
    b = b`;   					  /*b is a column vector */
    read all var{g11 g12 g13} into g1;
    read all var{g21 g22 g23} into g2;
	read all var{g31 g32 g33} into g3;
	g = g1//g2//g3;
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02 s03} into s0;
	read all var{s1 s2 s3} into s;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2', 'Brand3'};
    brandsp = brands`;

    one = i(3);
   

    s0b = s0@b;              /*Note: b is a column vector and s is a row vector*/
    sp = s`;
    emij = (g - s0b)#(1/sp) - one;           /*Marshallian Elasticity old eta1*/
    eta = 1 + b/sp;                   /*Expenditure Elasticity old ee and eep*/
    etap = eta`;                   /* Expenditure Elasticity as a row old eep*/
    ehij = emij + s@eta;         /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);            /*Hicks-Allen measures of elasticity */


    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

   	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;
    print key;
    print chk1;
    *print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;
   

    start noprint;
	zzz = 5;  *dummy step for no printing module;
	finish;

    run &print;

    /*  save the marshallian elasticity
    create mpe from emij[colname = brands];
    append from emij;
    close mpe;
    */
    
    elasticities = shape(emij,1)||shape(ehij,1)||etap||shape(haij,1);;
    enames = {m11 m12 m13 m21 m22 m23 m31 m32 m33
              h11 h12 h13 h21 h22 h23 h31 h32 h33
              e1 e2 e3
              a11 a12 a13 a21 a22 a23 a31 a32 a33};
    create &matcat from elasticities[colname = enames];
    append from elasticities;
    close &matcat;


    RESET Storage=work.&matcat;
    remove _all_;
    store b g s s0 key brands emij ehij etap haij;

    quit;

    proc datasets lib = work nolist;
    delete coeffs varcov;
    run; quit;

	%MEND;




/*****************************************************************************/
/**************************** DEFINE MACRO ELASTICITY4B **********************/
/*****************************************************************************/

/*******************************************************************************

Name of Macro:          elasticity4b(bnames,results,atshares,model,matcat,print)


Purpose: The purpose of the macro is to compute the matrix of marshallian and
compensated elasticities for a segment with three (4) drugs in it.


Arguments of Macro:
(1) &bnames
    Datasset containing the names and descriptions of the drugs in this segment
    e.g. namesegment110 etc.)

(2) &results
    Name of the dataset containing estimation results from AIDs model where the
    results are informat as stored by "PROC MODEL" procedure. Assumption
    is that the dataset will contain both the coefficients and the var-cov matrix.
    Additionally, the datset contains a column "_TYPE_" (= Type of Estimation) which
    states what type of estimation procedure was used (OLS, 2SLS, ITSUR etc.).
    Results from multiple estimation procedures may be stored in the same dataset.

(3) &atshares
    Name of the dataset containing the value of the shares (one per drug) at which
    elasticities should be computed (e.g. meanshares). Additionally, same dataset
    contains the values of the stone-weigts. 

(4) &model
    Argument indicating elasticities to be based on which model, i.e., ols, 2sls,
    etc. Note that the value of the macro must be stated in quotes and in upper case
    as in 'OLS' or '2SLS' or 'ITSUR' ect.

(5) &matcat
    Name of a SAS catalog where the results can be stored. Also, name of the temp
    SAS dataset where all results will be sent (in one row).

(6) &PRINT 
     Valid values are print or noprint. 

*********************************************************************************/


%MACRO ELASTICITY4B(BNAMES,RESULTS,ATSHARES,MODEL,MATCAT,PRINT);

    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2 b3 b4} into b;
    b = b`;   					  /*b is a column vector */
    read all var{g11 g12 g13 g14} into g1;
    read all var{g21 g22 g23 g24} into g2;
	read all var{g31 g32 g33 g34} into g3;
	read all var{g41 g42 g43 g44} into g4;
	g = g1//g2//g3//g4;
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02 s03 s04} into s0;
	read all var{s1 s2 s3 s4} into s;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2', 'Brand3', 'Brand4'};
    brandsp = brands`;

    one = i(4);
   

    s0b = s0@b;              /*Note: b is a column vector and s is a row vector*/
    sp = s`;
    emij = (g - s0b)#(1/sp) - one;           /*Marshallian Elasticity old eta1*/
    eta = 1 + b/sp;                   /*Expenditure Elasticity old ee and eep*/
    etap = eta`;                   /* Expenditure Elasticity as a row old eep*/
    ehij = emij + s@eta;         /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);            /*Hicks-Allen measures of elasticity */

    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;
    print key;
    print chk1;
    *print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;
   

    start noprint;
	zzz = 5;  *dummy step for no printing module;
	finish;

    run &print;

    /*  save the marshallian elasticity
    create mpe from emij[colname = brands];
    append from emij;
    close mpe;
    */
    
    elasticities = shape(emij,1)||shape(ehij,1)||etap||shape(haij,1);
    enames = {m11 m12 m13 m14 m21 m22 m23 m24 m31 m32 m33 m34 m41 m42 m43 m44
              h11 h12 h13 h14 h21 h22 h23 h24 h31 h32 h33 h34 h41 h42 h43 h44
              e1 e2 e3 e4 a11
              a12 a13 a14 a21 a22 a23 a24 a31 a32 a33 a34 a41 a42 a43 a44};
    create &matcat from elasticities[colname = enames];
    append from elasticities;
    close &matcat;


    RESET Storage=work.&matcat;
    remove _all_;
    store b g s s0 key brands emij ehij etap haij;

    quit;

    proc datasets lib = work nolist;
    delete coeffs varcov;
    run; quit;

	%MEND;





/*****************************************************************************/
/**************************** DEFINE MACRO ELASTICITY5B **********************/
/*****************************************************************************/

/*******************************************************************************

Name of Macro:          elasticity5b(bnames,results,atshares,model,matcat,print)


Purpose: The purpose of the macro is to compute the matrix of marshallian and
compensated elasticities for a segment with three (5) drugs in it.


Arguments of Macro:
(1) &bnames
    Datasset containing the names and descriptions of the drugs in this segment
    e.g. namesegment110 etc.)

(2) &results
    Name of the dataset containing estimation results from AIDs model where the
    results are informat as stored by "PROC MODEL" procedure. Assumption
    is that the dataset will contain both the coefficients and the var-cov matrix.
    Additionally, the datset contains a column "_TYPE_" (= Type of Estimation) which
    states what type of estimation procedure was used (OLS, 2SLS, ITSUR etc.).
    Results from multiple estimation procedures may be stored in the same dataset.

(3) &atshares
    Name of the dataset containing the value of the shares (one per drug) at which
    elasticities should be computed (e.g. meanshares). Additionally, same dataset
    contains the values of the stone-weigts. 

(4) &model
    Argument indicating elasticities to be based on which model, i.e., ols, 2sls,
    etc. Note that the value of the macro must be stated in quotes and in upper case
    as in 'OLS' or '2SLS' or 'ITSUR' ect.

(5) &matcat
    Name of a SAS catalog where the results can be stored. Also, name of the temp
    SAS dataset where all results will be sent (in one row).

(6) &PRINT 
     Valid values are print or noprint. 

*********************************************************************************/


%MACRO ELASTICITY5B(BNAMES,RESULTS,ATSHARES,MODEL,MATCAT,PRINT);

    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2 b3 b4 b5} into b;
    b = b`;   					  /*b is a column vector */
    read all var{g11 g12 g13 g14 g15} into g1;
    read all var{g21 g22 g23 g24 g25} into g2;
	read all var{g31 g32 g33 g34 g35} into g3;
	read all var{g41 g42 g43 g44 g45} into g4;
    read all var{g51 g52 g53 g54 g55} into g5;
	g = g1//g2//g3//g4//g5;
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02 s03 s04 s05} into s0;
	read all var{s1 s2 s3 s4 s5} into s;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2', 'Brand3', 'Brand4', 'Brand5'};
    brandsp = brands`;

    one = i(5);
   

    s0b = s0@b;              /*Note: b is a column vector and s is a row vector*/
    sp = s`;
    emij = (g - s0b)#(1/sp) - one;           /*Marshallian Elasticity old eta1*/
    eta = 1 + b/sp;                   /*Expenditure Elasticity old ee and eep*/
    etap = eta`;                   /* Expenditure Elasticity as a row old eep*/
    ehij = emij + s@eta;         /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);            /*Hicks-Allen measures of elasticity */

    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

  	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;
    print key;
    print chk1;
    *print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;
   

    start noprint;
	zzz = 5;  *dummy step for no printing module;
	finish;

    run &print;

    /*  save the marshallian elasticity
    create mpe from emij[colname = brands];
    append from emij;
    close mpe;
    */
    
    elasticities = shape(emij,1)||shape(ehij,1)||etap||shape(haij,1);
    enames = {m11 m12 m13 m14 m15 m21 m22 m23 m24 m25 m31 m32 m33 m34 m35 m41 m42 m43 m44 m45 m51 m52 m53 m54 m55
              h11 h12 h13 h14 h15 h21 h22 h23 h24 h25 h31 h32 h33 h34 h35 h41 h42 h43 h44 h45 h51 h52 h53 h54 h55
              e1 e2 e3 e4 e5
              a11 a12 a13 a14 a15 a21 a22 a23 a24 a25 a31 a32 a33 a34 a35 a41 a42 a43 a44 a45 a51 a52 a53 a54 a55};
    create &matcat from elasticities[colname = enames];
    append from elasticities;
    close &matcat;


    RESET Storage=work.&matcat;
    remove _all_;
    store b g s s0 key brands emij ehij etap haij;

    quit;

    proc datasets lib = work nolist;
    delete coeffs varcov;
    run; quit;

	%MEND;







/*****************************************************************************/
/**************************** DEFINE MACRO ELASTICITY7B **********************/
/*****************************************************************************/

/*******************************************************************************

Name of Macro:          elasticity7b(bnames,results,atshares,model,matcat,print)


Purpose: The purpose of the macro is to compute the matrix of marshallian and
compensated elasticities for a segment with seven (7) drugs in it.


Arguments of Macro:
(1) &bnames
    Datasset containing the names and descriptions of the drugs in this segment
    e.g. namesegment116 etc.)

(2) &results
    Name of the dataset containing estimation results from AIDs model where the
    results are informat as stored by "PROC MODEL" procedure. Assumption
    is that the dataset will contain both the coefficients and the var-cov matrix.
    Additionally, the datset contains a column "_TYPE_" (= Type of Estimation) which
    states what type of estimation procedure was used (OLS, 2SLS, ITSUR etc.).
    Results from multiple estimation procedures may be stored in the same dataset.

(3) &atshares
    Name of the dataset containing the value of the shares (one per drug) at which
    elasticities should be computed (e.g. meanshares). Additionally, same dataset
    contains the values of the stone-weigts. 

(4) &model
    Argument indicating elasticities to be based on which model, i.e., ols, 2sls,
    etc. Note that the value of the macro must be stated in quotes and in upper case
    as in 'OLS' or '2SLS' or 'ITSUR' ect.

(5) &matcat
    Name of a SAS catalog where the results can be stored. Also, name of the temp
    SAS dataset where all results will be sent (in one row).

(6) &PRINT 
     Valid values are print or noprint. 

*********************************************************************************/


%MACRO ELASTICITY7B(BNAMES,RESULTS,ATSHARES,MODEL,MATCAT,PRINT);

    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

    proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{b1 b2 b3 b4 b5 b6 b7} into b;
    b = b`;   					  /*b is a column vector */
    read all var{g11 g12 g13 g14 g15 g16 g17} into g1;
    read all var{g21 g22 g23 g24 g25 g26 g27} into g2;
	read all var{g31 g32 g33 g34 g35 g36 g37} into g3;
	read all var{g41 g42 g43 g44 g45 g46 g47} into g4;
    read all var{g51 g52 g53 g54 g55 g56 g57} into g5;
    read all var{g61 g62 g63 g64 g65 g66 g67} into g6;
    read all var{g71 g72 g73 g74 g75 g76 g77} into g7;
	g = g1//g2//g3//g4//g5//g6//g7;
    close coeffs;

    use &bnames;
    read all var{description} into key;
    close &bnames;

    use &atshares;
    read all var{s01 s02 s03 s04 s05 s06 s07} into s0;
	read all var{s1 s2 s3 s4 s5 s6 s7} into s;
    close &atshares;

	
    brands  = {'Brand1', 'Brand2', 'Brand3', 'Brand4', 'Brand5', 'Brand6', 'Brand7'};
    brandsp = brands`;

    one = i(7);
   

    s0b = s0@b;              /*Note: b is a column vector and s is a row vector*/
    sp = s`;
    emij = (g - s0b)#(1/sp) - one;           /*Marshallian Elasticity old eta1*/
    eta = 1 + b/sp;                   /*Expenditure Elasticity old ee and eep*/
    etap = eta`;                   /* Expenditure Elasticity as a row old eep*/
    ehij = emij + s@eta;         /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);            /*Hicks-Allen measures of elasticity */

    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

  	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;
    print key;
    print chk1;
    *print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;
   

    start noprint;
	zzz = 5;  *dummy step for no printing module;
	finish;

    run &print;

    /*  save the marshallian elasticity
    create mpe from emij[colname = brands];
    append from emij;
    close mpe;
    */
    
    elasticities = shape(emij,1)||shape(ehij,1)||etap||shape(haij,1);
    enames = {m11 m12 m13 m14 m15 m16 m17
              m21 m22 m23 m24 m25 m26 m27
              m31 m32 m33 m34 m35 m36 m37
              m41 m42 m43 m44 m45 m46 m47
              m51 m52 m53 m54 m55 m56 m57
              m61 m62 m63 m64 m65 m66 m67
              m71 m72 m73 m74 m75 m76 m77
              h11 h12 h13 h14 h15 h16 h17
              h21 h22 h23 h24 h25 h26 h27
              h31 h32 h33 h34 h35 h36 h37
              h41 h42 h43 h44 h45 h46 h47
              h51 h52 h53 h54 h55 h56 h57
              h61 h62 h63 h64 h65 h66 h67
              h71 h72 h73 h74 h75 h76 h77
              e1 e2 e3 e4 e5 e6 e7
              a11 a12 a13 a14 a15 a16 a17
              a21 a22 a23 a24 a25 a26 a27
              a31 a32 a33 a34 a35 a36 a37
              a41 a42 a43 a44 a45 a46 a47
              a51 a52 a53 a54 a55 a56 a57
              a61 a62 a63 a64 a65 a66 a67
              a71 a72 a73 a74 a75 a76 a77
              };
    create &matcat from elasticities[colname = enames];
    append from elasticities;
    close &matcat;


    RESET Storage=work.&matcat;
    remove _all_;
    store b g s s0 key brands emij ehij etap haij;

    quit;

    proc datasets lib = work nolist;
    delete coeffs varcov;
    run; quit;

	%MEND;






/*****************************************************************************/
/**************************** DEFINE MACRO ELASTICITY4BLNQ ******************/
/*****************************************************************************/

/*******************************************************************************

Name of Macro:          ELASTICITY4BLNQ(bnames,results,atshares,model,matcat,print)


Purpose: The purpose of the macro is to load the regression parameters (from
	proc model output) into a SAS catalog (with matrics) for a model of the form 
	lnq1 = a1+ b1*ly + g11*lp1 + g12*lp2 + g13*lp3 + g14*lp4 + t12*yr02 + t13*yr03
	...
	...
    lnq4 = a4 + b4*ly + g41*lp1 + g42*lp2 + g43*lp3 + g44*lp4 + t42*yr02 + t43*yr03


The elasticities are computed trivially since they are equal to the coefficients. 


Arguments of Macro:
(1) &bnames
    Datasset containing the names and descriptions of the drugs in this segment
    e.g. namesegment110 etc.)

(2) &results
    Name of the dataset containing estimation results from AIDs model where the
    results are informat as stored by "PROC MODEL" procedure. Assumption
    is that the dataset will contain both the coefficients and the var-cov matrix.
    Additionally, the datset contains a column "_TYPE_" (= Type of Estimation) which
    states what type of estimation procedure was used (OLS, 2SLS, ITSUR etc.).
    Results from multiple estimation procedures may be stored in the same dataset.

(3) &atshares
    Name of the dataset containing the value of the shares (one per drug) at which
    elasticities should be computed (e.g. meanshares). Additionally, same dataset
    contains the values of the stone-weigts. 

(4) &model
    Argument indicating elasticities to be based on which model, i.e., ols, 2sls,
    etc. Note that the value of the macro must be stated in quotes and in upper case
    as in 'OLS' or '2SLS' or 'ITSUR' ect.

(5) &matcat
    Name of a SAS catalog where the results can be stored. Also, name of the temp
    SAS dataset where all results will be sent (in one row).

(6) &PRINT 
     Valid values are print or noprint. 

*********************************************************************************/


%MACRO elasticity4Blnq(bnames,results,atshares,model,matcat,print);


    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

	proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{g11 g12 g13 g14} into g1;
    read all var{g21 g22 g23 g24} into g2;
    read all var{g31 g32 g33 g34} into g3;
    read all var{g41 g42 g43 g44} into g4; 
    read all var{b1 b2 b3 b4} into b;
    g = g1//g2//g3//g4;
    b = b`;
    close coeffs;


    use &atshares;
    read all var{s01 s02 s03 s04} into s0;
	read all var{s1 s2 s3 s4} into s;
    close &atshares;


    use &bnames;
    read all var{description} into key;
    close &bnames;


	brands  = {'Brand1', 'Brand2', 'Brand3', 'Brand4'};
    brandsp = brands`;

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;


    sp = s`;
    emij = g ;           /*Marshallian Elasticity old eta1*/
    eta =  b;                   /*Expenditure Elasticity old ee and eep*/
    etap = eta`;                   /* Expenditure Elasticity as a row old eep*/
    ehij = emij + s@eta;         /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);            /*Hicks-Allen measures of elasticity */


    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

  	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;
    print key;
    print chk1;
    *print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;
   

    start noprint;
	zzz = 5;  *dummy step for no printing module;
	finish;

    run &print;

    /*  save the marshallian elasticity
    create mpe from emij[colname = brands];
    append from emij;
    close mpe;
    */
    
    elasticities = shape(emij,1)||shape(ehij,1)||etap||shape(haij,1);
    enames = {m11 m12 m13 m14 m21 m22 m23 m24 m31 m32 m33 m34 m41 m42 m43 m44
              h11 h12 h13 h14 h21 h22 h23 h24 h31 h32 h33 h34 h41 h42 h43 h44
              e1 e2 e3 e4
              a11 a12 a13 a14 a21 a22 a23 a24 a31 a32 a33 a34 a41 a42 a43 a44};
    create &matcat from elasticities[colname = enames];
    append from elasticities;
    close &matcat;



    RESET Storage=work.&matcat;
    remove _all_;
    store b g s s0 key brands emij ehij etap haij ;

  

	quit;

    proc datasets lib = work nolist;
    delete coeffs varcov;
    run; quit;

%MEND;





/*****************************************************************************/
/**************************** DEFINE MACRO ELASTICITY3BLNQ ******************/
/*****************************************************************************/

/*******************************************************************************

Name of Macro:          ELASTICITY4BLNQ(bnames,results,atshares,model,matcat,print)


Purpose: The purpose of the macro is to load the regression parameters (from
	proc model output) into a SAS catalog (with matrics) for a model of the form 
	lnq1 = a1+ b1*ly + g11*lp1 + g12*lp2 + g13*lp3 +  t12*yr02 + t13*yr03
	...
	...
    lnq3 = a4 + b4*ly + g41*lp1 + g42*lp2 + g43*lp3 + t42*yr02 + t43*yr03


The elasticities are computed trivially since they are equal to the coefficients. 


Arguments of Macro:
(1) &bnames
    Datasset containing the names and descriptions of the drugs in this segment
    e.g. namesegment110 etc.)

(2) &results
    Name of the dataset containing estimation results from AIDs model where the
    results are informat as stored by "PROC MODEL" procedure. Assumption
    is that the dataset will contain both the coefficients and the var-cov matrix.
    Additionally, the datset contains a column "_TYPE_" (= Type of Estimation) which
    states what type of estimation procedure was used (OLS, 2SLS, ITSUR etc.).
    Results from multiple estimation procedures may be stored in the same dataset.

(3) &atshares
    Name of the dataset containing the value of the shares (one per drug) at which
    elasticities should be computed (e.g. meanshares). Additionally, same dataset
    contains the values of the stone-weigts. 

(4) &model
    Argument indicating elasticities to be based on which model, i.e., ols, 2sls,
    etc. Note that the value of the macro must be stated in quotes and in upper case
    as in 'OLS' or '2SLS' or 'ITSUR' ect.

(5) &matcat
    Name of a SAS catalog where the results can be stored. Also, name of the temp
    SAS dataset where all results will be sent (in one row).

(6) &PRINT 
     Valid values are print or noprint. 

*********************************************************************************/


%MACRO elasticity3Blnq(bnames,results,atshares,model,matcat,print);


    data coeffs (drop = _name_ _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ = '' then output;
    run; quit;

    data varcov (drop =  _type_ _status_ _nused_);
    set &results;
    if _type_ = &model and _NAME_ ~= '' then output;
    run; quit;

	proc iml;
    use coeffs;
    read all into coeffs[colname = varnames];
    read all var{g11 g12 g13} into g1;
    read all var{g21 g22 g23} into g2;
    read all var{g31 g32 g33} into g3;
    read all var{b1 b2 b3} into b;
    g = g1//g2//g3;
    b = b`;
    close coeffs;


    use &atshares;
    read all var{s01 s02 s03} into s0;
	read all var{s1 s2 s3} into s;
    close &atshares;


    use &bnames;
    read all var{description} into key;
    close &bnames;


	brands  = {'Brand1', 'Brand2', 'Brand3'};
    brandsp = brands`;

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;


    sp = s`;
    emij = g ;           /*Marshallian Elasticity old eta1*/
    eta =  b;                   /*Expenditure Elasticity old ee and eep*/
    etap = eta`;                   /* Expenditure Elasticity as a row old eep*/
    ehij = emij + s@eta;         /*Compensated (Hicksian) Elasticity old eta2*/
    haij = ehij#(1/s);            /*Hicks-Allen measures of elasticity */


    cournotagg = emij#sp;
    cournotagg = cournotagg[+,];
    engleagg = s*eta;
    hickslaw = ehij[,+];
    hickslaw = hickslaw`;
    chk1 = s//cournotagg//hickslaw;
    etap2 = etap||engleagg;

    etap2lab = brandsp||'SUM_i [ETA_i*S_i] = 1';

    mattrib emij label = {"Conditional Marshalian Price Elasticities (Em_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

    mattrib ehij label = {"Conditional Compensated Price Elasticities (Eh_ij) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;

  	mattrib haij label = {"Conditional Hicks-Allen Price Elasticities (Eh_ij/S_j) - Per &model"}
    format = 9.5 rowname = brands colname = brandsp;


    mattrib s label = {'Elasticities computed at the following Shares (S_i)'}
    format = 9.5 colname = brandsp;


    mattrib etap label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = brandsp;

    mattrib etap2 label = {"Expenditure Elasticities (ETA_i) - Per &model"}
    format = 9.5 colname = etap2lab rowname = 'ETA_i';

    mattrib key label = {'Brand Descriptions'}
    rowname = brands;

    mattrib chk1 label = {'Cournot Aggregation and Hicks Law Check'}
    format = 9.5 colname = brandsp
    rowname = {'Elasticities Computed at Shares (S_i)','SUM_j[Em_ij*S*j] = -S_i forall i','SUM_j[Eh_ij] = 0 forall i'};

    mattrib engleagg label = {'Engle Aggregation Check'}
    format = 9.5 rowname = {'SUM_i [ETA_i*S_i] = 1'};

	start print;
    print key;
    print chk1;
    *print s;
    print etap2;
    print emij ;
    print ehij  ;
	print haij;
    finish;
   

    start noprint;
	zzz = 5;  *dummy step for no printing module;
	finish;

    run &print;

    /*  save the marshallian elasticity
    create mpe from emij[colname = brands];
    append from emij;
    close mpe;
    */
    
    elasticities = shape(emij,1)||shape(ehij,1)||etap||shape(haij,1);
    enames = {m11 m12 m13 m21 m22 m23 m31 m32 m33
              h11 h12 h13 h21 h22 h23 h31 h32 h33
              e1 e2 e3
              a11 a12 a13 a21 a22 a23 a31 a32 a33};
    create &matcat from elasticities[colname = enames];
    append from elasticities;
    close &matcat;



    RESET Storage=work.&matcat;
    remove _all_;
    store b g s s0 key brands emij ehij etap haij ;

  

	quit;

    proc datasets lib = work nolist;
    delete coeffs varcov;
    run; quit;

%MEND;
